/*
troll.c - Slashdot trolls hack for RSStool

Copyright (c) 2006 NoisyB


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <time.h>
#include "misc/net.h"
#include "misc/string.h"
#include "misc/rss.h"
#include "misc/misc.h"
#include "rsstool_defines.h"
#include "rsstool.h"
#include "rsstool_misc.h"
#include "troll.h"


#define SLASHDOT_URL_S "http://slashdot.org"
#define SLASHDOT_RSS_HOST_S "http://rss.slashdot.org"
#define SLASHDOT_RSS_S "http://rss.slashdot.org/Slashdot/slashdot"
#define SLASHDOT_REQ_OLDEST_FIRST_S "/comments.pl?threshold=-1&mode=flat&commentsort=0&startat=%d&sid=%s"
#define SLASHDOT_REQ_NEWEST_FIRST_S "/comments.pl?threshold=-1&mode=flat&commentsort=1&startat=%d&sid=%s"
#define SLASHDOT_ARTICLE_LINK_HOOK  "slashdot.org/article.pl?sid=%s"


#if 0
static const char *
get_link (const char *str)
{
  char buf[MAXBUFSIZE];

  if (stristr (str, "item rdf:about="))
    {
      strncpy (buf, str, MAXBUFSIZE)[MAXBUFSIZE - 1];
      rsstool_set_link (rsstool.article_count, strtrim_s (buf, "about=\"", "&amp;"));

      rsstool.article_count++;
    }

  return "";
}


static const char *
get_title (const char *str)
{
  (void) str;
  static int count = 0;

  if (rsstool.article_count != count)
    rsstool_set_title (rsstool.article_count, stritrim_s (buffer, "<title>", "</title>"));

  count = rsstool.article_count;

  return "";
}


static const char *
get_desc (const char *str)
{
  (void) str;
  static int count = 0;

  if (rsstool.article_count != count)
    {
#if 1
      stritrim_s (buffer, "<description>", "</description>");
      rsstool_set_desc (rsstool.article_count, stritrim_s (buffer, "writes", NULL));
#else
      rsstool_set_desc (rsstool.article_count, stritrim_s (buffer, "<description>", "</description>"));
#endif
    }

  count = rsstool.article_count;

  return "";
}


static const char *
get_date (const char *str)
{
  (void) str;
  static int count = 0;

  if (rsstool.article_count != count)
    rsstool_set_date (rsstool.article_count, stritrim_s (buffer, "<dc:date>", "</dc:date>"));

  count = rsstool.article_count;

  return "";
}


st_tag_filter_t zophar_item_filter[] = {
  {
    "item",
    NULL,
    get_link
  },
  {
    "title",
    NULL,
    get_title
  },
  {
    "description",
    NULL,
    get_desc
  },
  {
    "dc:date",
    NULL,
    get_date
  },
  {NULL, NULL, NULL}
};


int
zophar_get_rss (st_rsstool_t *rt)
{
  st_net_t *client = NULL;
  st_strurl_t url;
  char *p = NULL;
  int cf = 0;

  if (!(client = net_init (0)))
    {
      fprintf (stderr, "ERROR: rsstool_rss_zophar()/net_init() failed\n");
      return -1;
    }

  strurl (&url, SLASHDOT_URL_S);

  if (net_open (client, url.host, 80) != 0)
    {
      fprintf (stderr, "ERROR: could not log on to %s\n", url.host);
      return -1;
    }

  p = net_build_http_request (SLASHDOT_URL_S, rsstool.user_agent, 0, NET_METHOD_GET, 0);
  net_write (client, (char *) p, strlen (p));

  memset (&item, 0, sizeof (st_rsstool_item_t));
  while (net_gets (client, buffer, MAXBUFSIZE))
    {
      p = strchr (buf, '\n');
      if (p)
        *p = 0;

      cf = net_tag_filter (buffer, zophar_item_filter, cf);

      if (*url && *title && *desc && date)
        {
          rsstool_add_item_s (rt, SLASHDOT_URL_S, item.date, item.url, item.title, item.desc);
          memset (&item, 0, sizeof (st_rsstool_item_t));
        }
    }

  net_quit (client);

  return 0;
}


const char *
slashdot_search_sid (st_net_t *client, const char *search)
{
  char *p = "xx/xx/xx/xxxxxxx.";

  if (!(*search))
    return "";

//#ifdef  DEBUG
  printf ("search: \"%s\"\n", search);
  fflush (stdout);
//#endif

  p = (char *) memmem2 (search, strlen (search), p, strlen (p), MEMCMP2_WCARD('x'));


  if (p)
    {
      *strchr (p, '.') = 0;
//#ifdef  DEBUG
  printf ("search2: \"%s\"\n", p);
  fflush (stdout);
//#endif
      return p;
    }

  return "";
}


const char *
slashdot_article_headline (st_net_t *client, const char *sid)
{
//TODO: return a string instead
  static char buf[MAXBUFSIZE];
  char article_title_s[MAXBUFSIZE];

  sprintf (article_title_s, SLASHDOT_ARTICLE_LINK_S, sid);

  while (net_gets (client, buf, MAXBUFSIZE))
    {
      char *p = strchr (buf, '\n');
      if (p)
        *p = 0;

      if (stristr (buf, article_title_s))  // the link to the slashdot article
        return buf;
    }

  return "";
}


int total_comment = 0;
int funny_comment = 0;
int troll_comment = 0;


int
slashdot_stats (st_net_t *server, int funny_flag)
{
  net_fprintf (server, "<br>Stats: %d comments, %d trolls (%d%%)",
    total_comment, troll_comment, misc_percent (troll_comment, total_comment));

  if (funny_flag)
    net_fprintf (server, ", %d funny (%d%%)",
      funny_comment, misc_percent (funny_comment, total_comment));

  net_fprintf (server, "<br>");

  return 0;
}



int
slashdot_comment_headline (st_net_t *client, char *buf, char *comment_title, int *total_comment)
{
          *comment_title = 0;
          if (stristr (buf, "<div class=\"title\">"))     // START of comment title
            {
              (*total_comment)++;

              strcat (comment_title, buf);

              while (net_gets (client, buf, MAXBUFSIZE))
                {
      char *p = strchr (buf, '\n');
      if (p)
        *p = 0;
                  strcat (comment_title, buf);

                  if (stristr (buf, "</div>"))    // END of comment title
                    break;
                }
            }
  return 0;
}


int
slashdot_comment_details (st_net_t *client, char *buf, char *comment_title)
{
          if (stristr (buf, "<div class=\"details\">"))     // START of comment details
            {
              strcat (comment_title, buf);

              while (net_gets (client, buf, MAXBUFSIZE))
                {
      char *p = strchr (buf, '\n');
      if (p)
        *p = 0;
                  strcat (comment_title, buf);

                  if (stristr (buf, "</div>"))    // END of comment details
                    break;
                }
            }
  return 0;
}


int
slashdot_get_trolls (st_net_t *client, const char *sid, st_net_t *server, int funny_flag, int direction)
{
  char buf[MAXBUFSIZE];
  char *p = NULL;
  int startat = 0;
  char comment_title[MAXBUFSIZE];
  int article_title = 0;

  if (!sid)
    return -1;

  if (!(*sid))
    return -1;

  total_comment = 0;
  funny_comment = 0;
  troll_comment = 0;

  while (total_comment > startat || !total_comment)
    {
      if (net_open (client, SLASHDOT_URL_S, 80) != 0)
        {
          fprintf (stderr, "ERROR: could not log on to " SLASHDOT_URL_S "\n");
          return -1;
        }

      startat = total_comment;        // startat 0 or last comment
      if (!direction)
        sprintf (buf, SLASHDOT_URL_S SLASHDOT_REQ_NEWEST_FIRST_S, startat, sid);
      else
        sprintf (buf, SLASHDOT_URL_S SLASHDOT_REQ_OLDEST_FIRST_S, startat, sid);
      p = net_build_http_request (buf, rsstool.user_agent, 0, NET_METHOD_GET, 0);
      net_write (client, (char *) p, strlen (p));

      // get link to article (only once per article)
      if (!article_title)
        {
          if ((p = slashdot_article_headline (client, sid)))
            net_fprintf (server, "Headline: %s<br><br><hr>", p);
          article_title = 1;
        }

      while (net_gets (client, buf, MAXBUFSIZE))
        {
      p = strchr (buf, '\n');
      if (p)
        *p = 0;
          slashdot_comment_headline (client, buf, comment_title, &total_comment);

          slashdot_comment_details (client, buf, comment_title);

          if (stristr (comment_title, "Flamebait)") ||
              stristr (comment_title, "Troll)") ||
              stristr (comment_title, "Offtopic)") ||
              stristr (comment_title, "Overrated)") ||
              (funny_flag && stristr (comment_title, "Funny)")) ||
              stristr (comment_title, "(Score:-1"))
          {

              if (funny_flag && stristr (comment_title, "Funny)"))
                funny_comment++;
              else
                troll_comment++;


              net_fprintf (server, "%s<br><br>", comment_title);  // NOW show the comment_title
              *comment_title = 0; // and clean it 

              while (net_gets (client, buf, MAXBUFSIZE))
                if (stristr (buf, "<div id=\"comment_body_"))       // START of comment body
                  {
                    net_puts (server, buf);

                    while (net_gets (client, buf, MAXBUFSIZE))
                      {
                        net_puts (server, buf);

                        if (stristr (buf, "</div>"))    // END of comment body
                          {
                            net_puts (server, "<br><hr>");
                            break;
                          }
                      }
                    break;
                  }
             }
         }

      net_close (client);
    }


  slashdot_stats (server, funny_flag);


  net_sync (server);

  return troll_comment + funny_comment;
}
#endif


int
troll_get_rss (st_rsstool_t *rt)
{
  (void) rt;
  st_rss_t *slashdot = NULL;
  int count = 0;
  int i = 0;
  char buf[MAXBUFSIZE];
  st_net_t *n = NULL;
  const char *p = NULL;
//  int cf = 0;

  p = net_http_get_to_temp (SLASHDOT_RSS_S, rsstool.user_agent, 0);
  if (!p)
    {
      fprintf (stderr, "ERROR: failed to download " SLASHDOT_RSS_S "\n");
      return -1;
    }

//  if (rss_demux (p) != 0)
//    return -1;

  slashdot = rss_open (p);
  if (!slashdot)
    return -1;

  count = rss_item_count (slashdot);
  if (!count)
    return -1;

  if (!(n = net_init (0)))
    {
      fprintf (stderr, "ERROR: n=net_init() failed\n");
      return -1;
    }

  if (net_open (n, SLASHDOT_URL_S, 80) != 0)
    {
      fprintf (stderr, "ERROR: could not log on to " SLASHDOT_URL_S "\n");
      return -1;
    }

  for (i = 0; i < count; i++)
    {
#if 0
      printf ("%s\n", slashdot->item[i].url);
#else
      p = net_build_http_request (SLASHDOT_RSS_S, rsstool.user_agent, 0, NET_METHOD_GET, 0);
      net_write (n, (char *) p, strlen (p));

  while (net_gets (n, buf, MAXBUFSIZE))
//    cf = net_tag_filter (buf, slashdot_rss_filter, cf);
    printf ("%s\n", buf);
#endif
    }

  net_quit (n);

  return 0;
}
